<?php

declare(strict_types=1);

namespace Drupal\mapsemble_store_locator\Entity;

use Drupal\Core\Entity\Attribute\ContentEntityType;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\ContentEntityDeleteForm;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityViewBuilder;
use Drupal\Core\Entity\Form\DeleteMultipleForm;
use Drupal\Core\Entity\Routing\DefaultHtmlRouteProvider;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\mapsemble_store_locator\Form\StoreForm;
use Drupal\mapsemble_store_locator\StoreInterface;
use Drupal\mapsemble_store_locator\StoreListBuilder;
use Drupal\user\EntityOwnerTrait;
use Drupal\views\EntityViewsData;

/**
 * Defines the store entity class.
 */
#[ContentEntityType(
    id: "mapsemble_store",
    label: new TranslatableMarkup("Mapsemble Store"),
    label_collection: new TranslatableMarkup("Mapsemble Stores"),
    label_singular: new TranslatableMarkup("Mapsemble store"),
    label_plural: new TranslatableMarkup("Mapsemble stores"),
    entity_keys: [
      "id" => "id",
      "label" => "label",
      "uuid" => "uuid",
      "owner" => "uid",
    ],
    handlers: [
      "list_builder" => StoreListBuilder::class,
      "views_data" => EntityViewsData::class,
      'view_builder' => EntityViewBuilder::class,
      "form" => [
        "add" => StoreForm::class,
        "edit" => StoreForm::class,
        "delete" => ContentEntityDeleteForm::class,
        "delete-multiple-confirm" => DeleteMultipleForm::class,
      ],
      "route_provider" => [
        'html' => DefaultHtmlRouteProvider::class,
      ],
    ],
    links: [
      "collection" => "/admin/content/mapsemble-store",
      "add-form" => "/admin/content/mapsemble/stores/add",
      "canonical" => "/admin/content/mapsemble/stores/{mapsemble_store}",
      "edit-form" => "/admin/content/mapsemble/stores/{mapsemble_store}/edit",
      "delete-form" => "/admin/content/mapsemble/stores/{mapsemble_store}/delete",
      "delete-multiple-form" => "/admin/content/mapsemble-store/delete-multiple",
    ],
    admin_permission: "administer mapsemble_store",
    base_table: "mapsemble_store",
    label_count: [
      "singular" => "@count store",
      "plural" => "@count stores",
    ],
    field_ui_base_route: "entity.mapsemble_store.settings"
)]
final class Store extends ContentEntityBase implements StoreInterface {
  use EntityChangedTrait;
  use EntityOwnerTrait;

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage): void {
    parent::preSave($storage);
    if (!$this->getOwnerId()) {
      // If no owner has been set explicitly, make the anonymous user the owner.
      $this->setOwnerId(0);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type): array {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['label'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Label'))
      ->setRequired(TRUE)
      ->setSetting('max_length', 255)
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['uid'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Author'))
      ->setSetting('target_type', 'user')
      ->setDefaultValueCallback(self::class . '::getDefaultEntityOwner')
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => 60,
          'placeholder' => '',
        ],
        'weight' => 15,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'author',
        'weight' => 15,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Authored on'))
      ->setDescription(t('The time that the store was created.'))
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'timestamp',
        'weight' => 20,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'datetime_timestamp',
        'weight' => 20,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the store was last edited.'));

    return $fields;
  }

}
